@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -38,12 +38,14 @@
 #include "qsoundeffect.h"
 
 #include <private/qdeclarativevideooutput_p.h>
+#include "qabstractvideofilter.h"
 
 #include "qdeclarativemultimediaglobal_p.h"
 #include "qdeclarativemediametadata_p.h"
 #include "qdeclarativeaudio_p.h"
 #include "qdeclarativeradio_p.h"
 #include "qdeclarativeradiodata_p.h"
+#include "qdeclarativeplaylist_p.h"
 #include "qdeclarativecamera_p.h"
 #include "qdeclarativecamerapreviewprovider_p.h"
 #include "qdeclarativecameraexposure_p.h"
@@ -57,6 +59,12 @@ QML_DECLARE_TYPE(QSoundEffect)
 
 QT_BEGIN_NAMESPACE
 
+static QObject *multimedia_global_object(QQmlEngine *qmlEngine, QJSEngine *jsEngine)
+{
+    Q_UNUSED(qmlEngine)
+    return new QDeclarativeMultimediaGlobal(jsEngine);
+}
+
 class QMultimediaDeclarativeModule : public QQmlExtensionPlugin
 {
     Q_OBJECT
@@ -90,7 +98,7 @@ public:
                                 trUtf8("CameraImageProcessing is provided by Camera"));
 
         // 5.2 types
-        qmlRegisterRevision<QDeclarativeVideoOutput, 2>(uri, 5, 2);
+        qmlRegisterType<QDeclarativeVideoOutput, 2>(uri, 5, 2, "VideoOutput");
 
         // 5.3 types
         // Nothing changed, but adding "import QtMultimedia 5.3" in QML will fail unless at
@@ -98,12 +106,23 @@ public:
         qmlRegisterType<QSoundEffect>(uri, 5, 3, "SoundEffect");
 
         // 5.4 types
-        qmlRegisterSingletonType(uri, 5, 4, "QtMultimedia", QDeclarativeMultimedia::initGlobalObject);
-        qmlRegisterRevision<QDeclarativeCamera, 1>(uri, 5, 4);
+        qmlRegisterSingletonType<QDeclarativeMultimediaGlobal>(uri, 5, 4, "QtMultimedia", multimedia_global_object);
+        qmlRegisterType<QDeclarativeCamera, 1>(uri, 5, 4, "Camera");
         qmlRegisterUncreatableType<QDeclarativeCameraViewfinder>(uri, 5, 4, "CameraViewfinder",
                                 trUtf8("CameraViewfinder is provided by Camera"));
 
+        // 5.5 types
+        qmlRegisterUncreatableType<QDeclarativeCameraImageProcessing, 1>(uri, 5, 5, "CameraImageProcessing", trUtf8("CameraImageProcessing is provided by Camera"));
+        qmlRegisterType<QDeclarativeCamera, 2>(uri, 5, 5, "Camera");
+
+        // 5.6 types
+        qmlRegisterType<QDeclarativeAudio, 1>(uri, 5, 6, "Audio");
+        qmlRegisterType<QDeclarativeAudio, 1>(uri, 5, 6, "MediaPlayer");
+        qmlRegisterType<QDeclarativePlaylist>(uri, 5, 6, "Playlist");
+        qmlRegisterType<QDeclarativePlaylistItem>(uri, 5, 6, "PlaylistItem");
+
         qmlRegisterType<QDeclarativeMediaMetaData>();
+        qmlRegisterType<QAbstractVideoFilter>();
     }
 
     void initializeEngine(QQmlEngine *engine, const char *uri)
