@@ -78,6 +78,8 @@ void tst_QAudioDecoderBackend::cleanup()
 void tst_QAudioDecoderBackend::fileTest()
 {
     QAudioDecoder d;
+    if (d.error() == QAudioDecoder::ServiceMissingError)
+        QSKIP("There is no audio decoding support on this platform.");
     QAudioBuffer buffer;
     quint64 duration = 0;
     int byteCount = 0;
@@ -260,6 +262,8 @@ void tst_QAudioDecoderBackend::fileTest()
 void tst_QAudioDecoderBackend::unsupportedFileTest()
 {
     QAudioDecoder d;
+    if (d.error() == QAudioDecoder::ServiceMissingError)
+        QSKIP("There is no audio decoding support on this platform.");
     QAudioBuffer buffer;
 
     QVERIFY(d.state() == QAudioDecoder::StoppedState);
@@ -336,6 +340,8 @@ void tst_QAudioDecoderBackend::unsupportedFileTest()
 void tst_QAudioDecoderBackend::corruptedFileTest()
 {
     QAudioDecoder d;
+    if (d.error() == QAudioDecoder::ServiceMissingError)
+        QSKIP("There is no audio decoding support on this platform.");
     QAudioBuffer buffer;
 
     QVERIFY(d.state() == QAudioDecoder::StoppedState);
@@ -408,6 +414,8 @@ void tst_QAudioDecoderBackend::corruptedFileTest()
 void tst_QAudioDecoderBackend::deviceTest()
 {
     QAudioDecoder d;
+    if (d.error() == QAudioDecoder::ServiceMissingError)
+        QSKIP("There is no audio decoding support on this platform.");
     QAudioBuffer buffer;
     quint64 duration = 0;
     int sampleCount = 0;
